module ImageMosaic
  module Image
    module Layout
      # Takes an array of URLs, returns a sliced array of Child images with sizes based on columns
      class Grid
        STARTING_Y = 0

        def initialize(items, width: WIDTH)
          @items = items
          @width = width
        end

        def items(y: STARTING_Y)
          [].tap do |images|
            rows.each do |row|
              x = 0
              row.each do |cell|
                images << cell.merge(y: y, x: x, h: child_dimension, w: child_dimension)
                x += child_dimension
              end
              y += child_dimension
            end
          end
        end

        private

        def rows
          @rows ||= valid_items.each_slice(columns).to_a
        end

        def columns
          Math.sqrt(valid_items.count).floor
        end

        def child_dimension
          @width / columns
        end

        def valid_items
          @valid_items ||= Data::Validator.new(@items).items
        end
      end
    end
  end
end
